package ru.landgrafhomyak.db.serdha0.api.table

import ru.landgrafhomyak.db.serdha0.api.misc.Column
import ru.landgrafhomyak.db.serdha0.api.misc.DatabaseType


public interface UpdateTableScope<tnUE : Any, toUE : Any> : CreateTableScope<tnUE> {
	public fun <RT, DT : DatabaseType<RT>> keepColumn(c: Column<RT, DT, toUE>): Column<RT, DT, tnUE>
	public fun <RT, DT : DatabaseType<RT>> keepColumnUntilUpgraded(c: Column<RT, DT, toUE>): Column<RT, DT, tnUE>
	public fun <RT, DT : DatabaseType<RT>> renameAndKeepColumn(c: Column<RT, DT, toUE>, newName: String): Column<RT, DT, tnUE>
	public fun deleteColumn(c: Column<*, *, toUE>)

	public fun keepIndex(i: Index<toUE>): Index<tnUE>
	public fun renameAndKeepIndex(i: Index<toUE>, newName: String): Index<tnUE>
	public fun deleteIndex(i: Index<toUE>)

	public fun keepCheck(i: CheckConstraint<toUE>): CheckConstraint<tnUE>
	public fun renameAndKeepCheck(i: CheckConstraint<toUE>, newName: String): CheckConstraint<tnUE>
	public fun deleteCheck(i: CheckConstraint<toUE>)

	public fun keepUniqueIndex(i: UniqueIndex<toUE>): UniqueIndex<tnUE>
	public fun renameAndKeepUniqueIndex(i: UniqueIndex<toUE>, newName: String): UniqueIndex<tnUE>
	public fun deleteUniqueIndex(i: UniqueIndex<toUE>)

	public fun <RT, DT : DatabaseType<RT>> keepDefaultValue(c: Column<RT, DT, tnUE>, d: DefaultConstraint<toUE>): DefaultConstraint<tnUE>
	public fun <RT, DT : DatabaseType<RT>> changeDefaultValue(c: Column<RT, DT, tnUE>, d: DefaultConstraint<toUE>): DefaultConstraint<tnUE>
	public fun deleteDefaultValue(i: DefaultConstraint<toUE>)

	public fun <ttUE : Any> keepForeignRowReference(
		frr: ForeignRowReference<toUE, ttUE>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<tnUE, ttUE>

	public fun <ttoUE : Any, ttnUE : Any> keepForeignRowReferenceToUpdatedTable(
		frr: ForeignRowReference<toUE, ttoUE>,
		updatedTable: Table<ttnUE, ttoUE>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<tnUE, ttnUE>

	public fun <ttUE : Any> renameAndKeepForeignRowReference(
		frr: ForeignRowReference<toUE, ttUE>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<tnUE, ttUE>

	public fun <ttoUE : Any, ttnUE : Any> renameAndKeepForeignRowReferenceToUpdatedTable(
		frr: ForeignRowReference<toUE, ttoUE>,
		updatedTable: Table<ttnUE, ttoUE>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<tnUE, ttnUE>

	public fun deleteForeignRowReference(frr: ForeignRowReference<toUE, *>)

	public fun <ttUE : Any> keepForeignRowReferenceUntilUpgraded(
		frr: ForeignRowReference<toUE, ttUE>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<tnUE, ttUE>

	public fun <ttoUE : Any, ttnUE : Any> keepForeignRowReferenceToUpdatedTableUntilUpgraded(
		frr: ForeignRowReference<toUE, ttoUE>,
		updatedTable: Table<ttnUE, ttoUE>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<tnUE, ttnUE>
}